const std = @import("std");
const builtin = @import("builtin");
const clap = @import("clap");

const craftflut = @import("craftflut");

pub const std_options: std.Options = .{
    .log_level = if (builtin.mode == .Debug) .debug else .info,
};

pub fn main() !void {
    var gpa = std.heap.GeneralPurposeAllocator(.{
        .thread_safe = true,
    }){};
    defer _ = gpa.deinit();
    const allocator = gpa.allocator();

    {
        const params = comptime clap.parseParamsComptime(
            \\-h, --help                Display this help and exit.
            \\--unix-socket <str>       Path to Unix socket to send block updates to.
            \\--queue <usize>           Block update queue capacity.
            \\--web-port <u16>          Web server port.
            \\--web-threads <u8>        Web server thread count.
        );

        var diag = clap.Diagnostic{};
        var res = clap.parse(clap.Help, &params, clap.parsers.default, .{
            .diagnostic = &diag,
            .allocator = allocator,
        }) catch |err| {
            diag.report(std.io.getStdErr().writer(), err) catch {};
            return err;
        };
        defer res.deinit();

        if (res.args.help != 0 or
            res.args.@"unix-socket" == null or
            res.args.@"web-port" == null)
        {
            const writer = std.io.getStdErr().writer();
            try writer.writeAll("craftflut\n");
            return clap.help(writer, clap.Help, &params, .{});
        }

        const config = craftflut.Config{
            .unix_socket_path = res.args.@"unix-socket".?,
            .dispatcher = .{
                .capacity = res.args.queue orelse 1024,
            },
            .web = .{
                .port = res.args.@"web-port".?,
                .threads = res.args.@"web-threads" orelse 1,
            },
        };
        try craftflut.start(allocator, &config);
    }
}
