const std = @import("std");

const models = @import("./models/root.zig");
const msg_queue = @import("msg_queue/root.zig");
const dispatchers = @import("dispatchers/root.zig");
const web = @import("web/root.zig");

const log = std.log.scoped(.craftflut);

pub const Config = struct {
    unix_socket_path: []const u8,
    dispatcher: dispatchers.BlockUpdate.Config,
    web: web.Config,
};

fn receiver(dispatcher: *dispatchers.BlockUpdate) void {
    dispatcher.start() catch unreachable;
}

pub fn start(allocator: std.mem.Allocator, config: *const Config) !void {
    log.info("Starting craftflut gateway", .{});

    log.info("Opening Unix socket: {s}", .{config.unix_socket_path});
    var stream = try std.net.connectUnixSocket(config.unix_socket_path);
    defer stream.close();

    var queue = try msg_queue.MsgQueueManaged(models.BlockUpdateMsg)
        .init(allocator, config.dispatcher.capacity);
    defer queue.deinit();

    var dispatcher = dispatchers.BlockUpdate.init(
        allocator,
        &config.dispatcher,
        queue.unmanaged(),
        stream,
    );
    log.info("Starting dispatcher", .{});
    _ = try std.Thread.spawn(.{}, receiver, .{&dispatcher});

    try web.start(allocator, &config.web, queue.unmanaged());
}
